@@ -19,13 +19,19 @@
 							gtk_tree_path_free(path);
 #define SCROLL_TO_ITER(model, view, iter)		GtkTreePath *path2 = gtk_tree_model_get_path(model, &iter); \
 							gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(view), path2, NULL, FALSE, 0, 0); \
-							gtk_tree_path_free(path2);							
+							gtk_tree_path_free(path2);
 #define EXPAND_ITER(model, view, iter)			GtkTreePath *path = gtk_tree_model_get_path(model, &iter); \
 							gtk_tree_view_expand_row(GTK_TREE_VIEW(view), path, FALSE); \
-							gtk_tree_path_free(path);							
+							gtk_tree_path_free(path);
 
 #include "main.h"
 
+enum {
+	PROGRAMMING_KEYPAD = 1,
+	HIDE_LEFT_KEYPAD = 2,
+	HIDE_RIGHT_KEYPAD = 4
+};
+
 struct mode_struct {
 	PrintOptions po;
 	EvaluationOptions eo;
@@ -34,12 +40,14 @@ struct mode_struct {
 	Number custom_output_base;
 	Number custom_input_base;
 	int precision;
-	string name;
+	std::string name;
 	bool rpn_mode;
 	bool interval;
 	bool adaptive_interval_display;
 	bool variable_units_enabled;
-	bool keypad;
+	int keypad;
+	bool autocalc;
+	bool complex_angle_form;
 };
 
 enum {
@@ -55,11 +63,91 @@ enum {
 	QALCULATE_HISTORY_OLD,
 	QALCULATE_HISTORY_REGISTER_MOVED,
 	QALCULATE_HISTORY_RPN_OPERATION,
-	QALCULATE_HISTORY_BOOKMARK
+	QALCULATE_HISTORY_BOOKMARK,
+	QALCULATE_HISTORY_MESSAGE
 };
 
-DECLARE_BUILTIN_FUNCTION(AnswerFunction)
-DECLARE_BUILTIN_FUNCTION(ExpressionFunction)
+enum {
+	SHORTCUT_TYPE_FUNCTION,
+	SHORTCUT_TYPE_FUNCTION_WITH_DIALOG,
+	SHORTCUT_TYPE_VARIABLE,
+	SHORTCUT_TYPE_UNIT,
+	SHORTCUT_TYPE_TEXT,
+	SHORTCUT_TYPE_DATE,
+	SHORTCUT_TYPE_VECTOR,
+	SHORTCUT_TYPE_MATRIX,
+	SHORTCUT_TYPE_SMART_PARENTHESES,
+	SHORTCUT_TYPE_CONVERT,
+	SHORTCUT_TYPE_CONVERT_ENTRY,
+	SHORTCUT_TYPE_OPTIMAL_UNIT,
+	SHORTCUT_TYPE_BASE_UNITS,
+	SHORTCUT_TYPE_OPTIMAL_PREFIX,
+	SHORTCUT_TYPE_TO_NUMBER_BASE,
+	SHORTCUT_TYPE_FACTORIZE,
+	SHORTCUT_TYPE_EXPAND,
+	SHORTCUT_TYPE_PARTIAL_FRACTIONS,
+	SHORTCUT_TYPE_SET_UNKNOWNS,
+	SHORTCUT_TYPE_RPN_UP,
+	SHORTCUT_TYPE_RPN_DOWN,
+	SHORTCUT_TYPE_RPN_SWAP,
+	SHORTCUT_TYPE_RPN_COPY,
+	SHORTCUT_TYPE_RPN_LASTX,
+	SHORTCUT_TYPE_RPN_DELETE,
+	SHORTCUT_TYPE_RPN_CLEAR,
+	SHORTCUT_TYPE_META_MODE,
+	SHORTCUT_TYPE_OUTPUT_BASE,
+	SHORTCUT_TYPE_INPUT_BASE,
+	SHORTCUT_TYPE_EXACT_MODE,
+	SHORTCUT_TYPE_DEGREES,
+	SHORTCUT_TYPE_RADIANS,
+	SHORTCUT_TYPE_GRADIANS,
+	SHORTCUT_TYPE_FRACTIONS,
+	SHORTCUT_TYPE_MIXED_FRACTIONS,
+	SHORTCUT_TYPE_SCIENTIFIC_NOTATION,
+	SHORTCUT_TYPE_SIMPLE_NOTATION,
+	SHORTCUT_TYPE_RPN_MODE,
+	SHORTCUT_TYPE_AUTOCALC,
+	SHORTCUT_TYPE_PROGRAMMING,
+	SHORTCUT_TYPE_KEYPAD,
+	SHORTCUT_TYPE_HISTORY,
+	SHORTCUT_TYPE_HISTORY_SEARCH,
+	SHORTCUT_TYPE_CONVERSION,
+	SHORTCUT_TYPE_STACK,
+	SHORTCUT_TYPE_MINIMAL,
+	SHORTCUT_TYPE_MANAGE_VARIABLES,
+	SHORTCUT_TYPE_MANAGE_FUNCTIONS,
+	SHORTCUT_TYPE_MANAGE_UNITS,
+	SHORTCUT_TYPE_MANAGE_DATA_SETS,
+	SHORTCUT_TYPE_STORE,
+	SHORTCUT_TYPE_NEW_VARIABLE,
+	SHORTCUT_TYPE_NEW_FUNCTION,
+	SHORTCUT_TYPE_PLOT,
+	SHORTCUT_TYPE_NUMBER_BASES,
+	SHORTCUT_TYPE_FLOATING_POINT,
+	SHORTCUT_TYPE_CALENDARS,
+	SHORTCUT_TYPE_PERCENTAGE_TOOL,
+	SHORTCUT_TYPE_PERIODIC_TABLE,
+	SHORTCUT_TYPE_UPDATE_EXRATES,
+	SHORTCUT_TYPE_COPY_RESULT,
+	SHORTCUT_TYPE_SAVE_IMAGE,
+	SHORTCUT_TYPE_HELP,
+	SHORTCUT_TYPE_QUIT
+};
+
+struct keyboard_shortcut {
+	guint key;
+	guint modifier;
+	int type;
+	std::string value;
+};
+
+std::string shortcut_to_text(guint key, guint state);
+const gchar *shortcut_type_text(int type);
+void update_accels();
+
+DECLARE_BUILTIN_FUNCTION(AnswerFunction, 0)
+DECLARE_BUILTIN_FUNCTION(ExpressionFunction, 0)
+DECLARE_BUILTIN_FUNCTION(SetTitleFunction, 0)
 
 class ViewThread : public Thread {
 protected:
@@ -76,17 +164,21 @@ protected:
 	virtual void run();
 };
 
-bool string_is_less(string str1, string str2);
+bool string_is_less(std::string str1, std::string str2);
 
 bool can_display_unicode_string_function(const char *str, void *w);
 bool can_display_unicode_string_function_exact(const char *str, void *w);
 void set_unicode_buttons();
 void set_operator_symbols();
 
-cairo_surface_t *draw_structure(MathStructure &m, PrintOptions po = default_print_options, InternalPrintStruct ips = top_ips, gint *point_central = NULL, int scaledown = 0, GdkRGBA *color = NULL);
+bool update_window_title(const char *str = NULL, bool is_result = false);
+
+cairo_surface_t *draw_structure(MathStructure &m, PrintOptions po = default_print_options, bool caf = false, InternalPrintStruct ips = top_ips, gint *point_central = NULL, int scaledown = 0, GdkRGBA *color = NULL, gint *x_offset = NULL, gint *w_offset = NULL, gint max_width = -1);
 
 void update_status_text();
 
+void clearresult();
+
 void set_result_size_request();
 void set_expression_size_request();
 
@@ -127,15 +219,15 @@ void on_tDataProperties_selection_changed(GtkTreeSelection *treeselection, gpoin
 void on_tUnitSelector_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
 void on_tUnitSelectorCategories_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
 
-void execute_expression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, string execute_str = string(), string str = string(), bool check_exrates = true);
-void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, string transformation = "", size_t stack_index = 0, bool register_moved = false);
+void execute_expression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, std::string execute_str = std::string(), std::string str = std::string(), bool check_exrates = true);
+void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, std::string transformation = "", size_t stack_index = 0, bool register_moved = false, bool supress_dialog = false);
 
 void set_rpn_mode(bool b);
 void calculateRPN(int op);
 void calculateRPN(MathFunction *f);
-void RPNRegisterAdded(string text, gint index = 0);
+void RPNRegisterAdded(std::string text, gint index = 0);
 void RPNRegisterRemoved(gint index);
-void RPNRegisterChanged(string text, gint index);
+void RPNRegisterChanged(std::string text, gint index);
 
 void result_display_updated();
 void result_format_updated();
@@ -217,7 +309,7 @@ bool edit_argument(Argument *arg = NULL);
 void edit_unknown(const char *category = "", Variable *v = NULL, GtkWidget *win = NULL);
 void edit_variable(const char *category = "", Variable *v = NULL, MathStructure *mstruct_ = NULL, GtkWidget *win = NULL);
 void edit_matrix(const char *category = "", Variable *v = NULL, MathStructure *mstruct_ = NULL, GtkWidget *win = NULL, gboolean create_vector = FALSE);
-void insert_matrix(const MathStructure *initial_value = NULL, GtkWidget *win = NULL, gboolean create_vector = FALSE, bool is_text_struct = false, bool is_result = false);
+void insert_matrix(const MathStructure *initial_value = NULL, GtkWidget *win = NULL, gboolean create_vector = FALSE, bool is_text_struct = false, bool is_result = false, GtkEntry *entry = NULL);
 void import_csv_file(GtkWidget *win = NULL);
 void export_csv_file(KnownVariable *v = NULL, GtkWidget *win = NULL);
 
@@ -228,24 +320,31 @@ void edit_names(ExpressionItem *item = NULL, const gchar *namestr = NULL, GtkWid
 
 gchar *font_name_to_css(const char *font_name);
 
-void reload_history();
+void reload_history(gint from_index = -1);
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+void hide_tooltip(GtkWidget*);
+
+void on_tShortcuts_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
+void on_tShortcutsType_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
+
 void insert_left_shift();
 void insert_right_shift();
 void insert_bitwise_and();
 void insert_bitwise_or();
 void insert_bitwise_xor();
 void insert_bitwise_not();
+void insert_angle_symbol();
 
 void update_mb_fx_menu();
 void update_mb_sto_menu();
 void update_mb_units_menu();
 void update_mb_pi_menu();
 void update_mb_to_menu();
+void update_mb_angles(AngleUnit angle_unit);
 
 void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewColumn*, gpointer);
 void on_units_convert_view_row_activated(GtkTreeView*, GtkTreePath *path, GtkTreeViewColumn*, gpointer);
@@ -258,7 +357,7 @@ void on_message_bar_response(GtkInfoBar *w, gint response_id, gpointer);
 void on_expressiontext_populate_popup(GtkTextView *w, GtkMenu *menu, gpointer user_data);
 void on_combobox_base_changed(GtkComboBox *w, gpointer user_data);
 void on_combobox_numerical_display_changed(GtkComboBox *w, gpointer user_data);
-void on_combobox_fraction_mode_changed(GtkComboBox *w, gpointer user_data);
+void on_button_fraction_toggled(GtkToggleButton *w, gpointer user_data);
 void on_button_exact_toggled(GtkToggleButton *w, gpointer user_data);
 void on_expander_keypad_expanded(GObject *o, GParamSpec *param_spec, gpointer user_data);
 void on_expander_history_expanded(GObject *o, GParamSpec *param_spec, gpointer user_data);
@@ -272,6 +371,7 @@ void on_menu_item_meta_mode_delete_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_quit_activate(GtkMenuItem *w, gpointer user_data);
 void on_colorbutton_status_error_color_color_set(GtkColorButton *w, gpointer user_data);
 void on_colorbutton_status_warning_color_color_set(GtkColorButton *w, gpointer user_data);
+void on_preferences_checkbutton_persistent_keypad_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_copy_separator_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_lower_case_numbers_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_e_notation_toggled(GtkToggleButton *w, gpointer user_data);
@@ -382,6 +482,7 @@ void on_menu_item_new_vector_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_function_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_dataset_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_unit_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_autocalc_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_rpn_mode_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_rpn_syntax_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_limit_implicit_multiplication_activate(GtkMenuItem *w, gpointer user_data);
@@ -396,6 +497,8 @@ void on_menu_item_degrees_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_radians_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_gradians_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_no_default_angle_unit_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_read_precision_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_rpn_syntax_activate(GtkMenuItem *w, gpointer user_data);
 void set_output_base_from_dialog(int base);
 void output_base_updated_from_menu();
 void input_base_updated_from_menu();
@@ -480,9 +583,12 @@ void on_popup_menu_item_view_vector_activate(GtkMenuItem *w, gpointer user_data)
 void on_popup_menu_item_complex_rectangular_activate(GtkMenuItem *w, gpointer user_data);
 void on_popup_menu_item_complex_exponential_activate(GtkMenuItem *w, gpointer user_data);
 void on_popup_menu_item_complex_polar_activate(GtkMenuItem *w, gpointer user_data);
+void on_popup_menu_item_complex_angle_activate(GtkMenuItem *w, gpointer user_data);
+void on_popup_menu_item_persistent_keypad_toggled(GtkCheckMenuItem *w, gpointer user_data);
 void on_menu_item_complex_rectangular_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_complex_exponential_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_complex_polar_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_complex_angle_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_normal_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_engineering_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_scientific_activate(GtkMenuItem *w, gpointer user_data);
@@ -547,6 +653,8 @@ void on_stackview_selection_changed(GtkTreeSelection *treeselection, gpointer us
 void on_stackview_item_edited(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer user_data);
 void on_stackview_item_editing_started(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path, gpointer user_data);
 void on_stackview_item_editing_canceled(GtkCellRenderer *renderer, gpointer user_data);
+void on_stackstore_row_inserted(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data);
+void on_stackstore_row_deleted(GtkTreeModel *model, GtkTreePath *path, gpointer user_data);
 
 void on_historyview_selection_changed(GtkTreeSelection *select, gpointer);
 
@@ -613,6 +721,10 @@ void on_nbases_entry_duo_changed(GtkEditable *editable, gpointer user_data);
 void on_nbases_entry_roman_changed(GtkEditable *editable, gpointer user_data);
 void on_nbases_entry_sexa_changed(GtkEditable *editable, gpointer user_data);
 
+void on_fp_entry_dec_changed(GtkEditable *editable, gpointer user_data);
+void on_fp_buffer_bin_changed(GtkTextBuffer *w, gpointer user_data);
+void on_fp_entry_hex_changed(GtkEditable *editable, gpointer user_data);
+
 void on_percentage_button_clear_clicked(GtkWidget*, gpointer);
 
 void on_menu_item_show_percentage_dialog_activate(GtkMenuItem *w, gpointer user_data);
@@ -681,6 +793,8 @@ void on_csv_export_radiobutton_matrix_toggled(GtkToggleButton *w, gpointer user_
 
 void on_type_label_date_clicked(GtkButton *w, gpointer user_data);
 void on_type_label_file_clicked(GtkButton *w, gpointer user_data);
+void on_type_label_vector_clicked(GtkButton *w, gpointer user_data);
+void on_type_label_matrix_clicked(GtkButton *w, gpointer user_data);
 
 void on_functions_button_deactivate_clicked(GtkButton *w, gpointer user_data);
 void on_variables_button_deactivate_clicked(GtkButton *w, gpointer user_data);
